import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { MatchEntity } from '../../../domain/match.entity';
import { MatchQuery } from './match.query';
import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';
import {
  Journey,
  JourneyProps,
} from '@modules/ad/core/domain/value-objects/journey.value-object';

export abstract class Algorithm {
  protected candidates: CandidateEntity[];
  protected selector: Selector;
  protected processors: Processor[];
  constructor(
    protected readonly query: MatchQuery,
    protected readonly repository: AdRepositoryPort,
  ) {}

  /**
   * Return Ads that matches the query as Matches
   */
  match = async (): Promise<MatchEntity[]> => {
    this.candidates = await this.selector.select();
    for (const processor of this.processors) {
      this.candidates = await processor.execute(this.candidates);
    }
    // console.log(JSON.stringify(this.candidates, null, 2));
    return this.candidates.map((candidate: CandidateEntity) =>
      MatchEntity.create({
        adId: candidate.id,
        role: candidate.getProps().role,
        distance: candidate.getProps().distance as number,
        duration: candidate.getProps().duration as number,
        journeys: candidate.getProps().journeys as Journey[],
      }),
    );
  };
}

/**
 * A selector queries potential candidates in a repository
 */
export abstract class Selector {
  protected readonly query: MatchQuery;
  protected readonly repository: AdRepositoryPort;
  constructor(query: MatchQuery, repository: AdRepositoryPort) {
    this.query = query;
    this.repository = repository;
  }
  abstract select(): Promise<CandidateEntity[]>;
}

/**
 * A processor processes candidates information
 */
export abstract class Processor {
  constructor(protected readonly query: MatchQuery) {}
  abstract execute(candidates: CandidateEntity[]): Promise<CandidateEntity[]>;
}
