import { Controller, Inject, UsePipes } from '@nestjs/common';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { ResponseBase, RpcValidationPipe } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { MatchPaginatedResponseDto } from '../dtos/match.paginated.response.dto';
import { QueryBus } from '@nestjs/cqrs';
import { MatchRequestDto } from './dtos/match.request.dto';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { AD_ROUTE_PROVIDER } from '@modules/ad/ad.di-tokens';
import { RouteProviderPort } from '@modules/ad/core/application/ports/route-provider.port';
import { Journey } from '@modules/ad/core/domain/value-objects/journey.value-object';
import { JourneyItem } from '@modules/ad/core/domain/value-objects/journey-item.value-object';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class MatchGrpcController {
  constructor(
    private readonly queryBus: QueryBus,
    @Inject(AD_ROUTE_PROVIDER)
    private readonly routeProvider: RouteProviderPort,
  ) {}

  @GrpcMethod('MatcherService', 'Match')
  async match(data: MatchRequestDto): Promise<MatchPaginatedResponseDto> {
    try {
      const matches: MatchEntity[] = await this.queryBus.execute(
        new MatchQuery(data, this.routeProvider),
      );
      return new MatchPaginatedResponseDto({
        data: matches.map((match: MatchEntity) => ({
          ...new ResponseBase(match),
          adId: match.getProps().adId,
          role: match.getProps().role,
          distance: match.getProps().distance,
          duration: match.getProps().duration,
          journeys: match.getProps().journeys.map((journey: Journey) => ({
            firstDate: journey.firstDate.toUTCString(),
            lastDate: journey.lastDate.toUTCString(),
            steps: journey.journeyItems.map((journeyItem: JourneyItem) => ({
              duration: journeyItem.duration,
              distance: journeyItem.distance as number,
              lon: journeyItem.lon,
              lat: journeyItem.lat,
              actors: journeyItem.actorTimes.map((actorTime: ActorTime) => ({
                role: actorTime.role,
                target: actorTime.target,
                firstDatetime: actorTime.firstMinDatetime.toUTCString(),
                firstMinDatetime: actorTime.firstMinDatetime.toUTCString(),
                firstMaxDatetime: actorTime.firstMaxDatetime.toUTCString(),
                lastDatetime: actorTime.lastDatetime.toUTCString(),
                lastMinDatetime: actorTime.lastMinDatetime.toUTCString(),
                lastMaxDatetime: actorTime.lastMaxDatetime.toUTCString(),
              })),
            })),
          })),
        })),
        page: 1,
        perPage: 5,
        total: matches.length,
      });
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
